#ifndef OPERATORINTERFACE_H
#define OPERATORINTERFACE_H

#include "CoTeXNode.h"


/// The base classe for the CoTeX operator
/**
 * An operator is an internal node of a CoTeXTree.
 * It joins two node (operators or primitives), and the inherited class
 * must implements two methods from CoTeXNode : 
 *  - getAttribute : must defined the attribute at this node of the tree, according
 *    with children's evaluation.
 *  - eval : the F-Rep evaluation for the node, not directly used in constructive
 *    texturing.   
 */
class OperatorInterface : public CoTeXNode {
public : 
  
  /// the Base Constructor, with only a name.
  OperatorInterface (QString name) : CoTeXNode (name) {}
  
  /// Validity as a CoTeXNode
  /**
   * Operators MUST BE internal nodes, and CoTeXTrees are complete. So operators must
   * have 2 children.
   */ 
  bool check () { return (getLeftChild () != NULL && getRightChild () != NULL);}
};


#endif // OPERATORINTERFACE_H

