#ifndef TEXTUREINTERFACE_H
#define TEXTUREINTERFACE_H 

#include <qstring.h>
#include <qmap.h>
#include <Vector3D.h>

#include "HyperFunable.h"
#include "Savable.h"
#include "AttributeInterface.h"

/// CoTeX texture interface
/**
 * Textures provide one main service : they give an attribute for an {x,y,z} position
 * in the space (as functions <code>f(x,y,z)</code>). This attribute can be a color, 
 * or space modifier, which encapsulate its own logic of application.
 *
 * @see #AttributeInterface
 */
class TextureInterface : public HyperFunable, public Savable {
public :
  
  /// Map for quering textures with their names
  typedef QMap<QString,TextureInterface*> TextureMap;
  
  /// he base constructeur
  /**
   * @param n the name of the texture, useful for registration, maps, and HyperFun.
   */
  TextureInterface (QString& n) : name (n) {}

  /// The destructor
  virtual ~TextureInterface () {}

  /// Returns the attribute's value at position {x,y,z} 
  /**
   *
   * @param position the position evaluated. 
   *
   * @return AttributeInterface* the generic attribute returned, 
   *                             which MUSTN'T BE DESTROYED by
   *                             other classes (this attribute is not 
   *                             in their responsabilities). (An 
   *                             old C-style cast can break the "const" security)
   *
   */
  virtual const AttributeInterface * operator() (const Vector3D & position) = 0;

  inline QString getName () const { return name;}
  void setName (QString & newName);

private :

  QString name;
   
};

#endif // TEXTUREINTERFACE_H
