#include "OperatorUnion.h"
#include <math.h>
#include <assert.h>
#include <StringTools.h>


static int unionCount = 0;
const int  UNION_SEQ_LEN = 4;

OperatorUnion::OperatorUnion () : OperatorInterface (StringTools::appendSequenceNumber ("Union", unionCount++, UNION_SEQ_LEN)) {
}

const AttributeInterface * OperatorUnion::getAttribute (const Vector3D &position){
  assert (check ());
  const AttributeInterface * res = getLeftChild ()->getAttribute (position);
  if (res != NULL)
    return res;
  return getRightChild()->getAttribute (position);
}

double OperatorUnion::eval (const Vector3D& position){
  double f1 = getLeftChild ()->eval (position);
  double f2 = getRightChild ()->eval (position);

  return f1+f2+sqrt(f1*f1+f2*f2);
}

// **********************
// Interface HyperFunable
// **********************
  
void OperatorUnion::writeHFString (QTextStream & outStream) {
  outStream << "my_model(x[3], a[1], s[3])" << endl << "{" << endl;
  writeDefArray (outStream);
  writeDefBody (outStream);
  outStream << endl << "}" << endl;
}


void OperatorUnion::writeDefArray (QTextStream & outStream) {
  getRightChild ()->writeDefArray (outStream);
  outStream << "array color_" << getName () << "[3];" << endl << endl;
  getLeftChild ()->writeDefArray (outStream);
  
}


void OperatorUnion::writeDefBody (QTextStream & outStream) {
  getRightChild ()->writeDefBody (outStream);
  getLeftChild ()->writeDefBody (outStream);
  QString lcName = getLeftChild ()->getName ();
  QString rcName = getRightChild ()->getName ();
  QString color_name;

  if (getFather () != NULL) 
    color_name = "color_" + getName ();
  else 
    color_name = "s";
  outStream << getName () <<" = hfA_Union(" << lcName << ", "<< rcName << ", " << color_name << ", ";
  outStream << "color_" << lcName << ", color_" << rcName << ", 1);" << endl;    
}


// *****************
// Interface Savable
// *****************

void OperatorUnion::save (QTextStream &stream) {
  stream << "operator union " << getName();
}

void OperatorUnion::load (QTextStream &stream) {
  QString n ;
  stream >> n ;
  setName (n) ;
}
