// --------------------------------------------------------------------------
//
// Reality
//
// --------------------------------------------------------------------------
//
// Sebastien Dedieu     (dedieu@labri.u-bordeaux.fr)
//
// LaBRI (Laboratoire Bordelais de Recherche en Informatique)
// Universit Bordeaux I - CNRS UMR 5800
// 351, cours de la Libration
// 33405 Talence Cedex
// France
//
// --------------------------------------------------------------------------


/*

  LinearSolver.cpp
  Aout 1999

*/


#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include <assert.h>
#include "LinearSolver.h"


static inline void nrerror(char *str)
{
  cerr << str << endl;
  exit(EXIT_FAILURE);
}


static inline double *vector(int low, int high)
{
  double *v=(double *)new double[low+high];
  assert(v!=0);

  return v;
}


static inline void free_vector(double *v, int, int)
{
  assert(v!=0);

  delete [] v;
}


static inline double SIGN(double a, double b)
{
  assert(a >= 0);

  if ( b >= 0)
    return a;

  return -a;
}


static inline double SQR(double a)
{
  return a*a;
}


static inline double FMAX(double a, double b)
{
  if (a>=b)
    return a;

  return b;
}


static inline int IMIN(int a, int b)
{
  if (a>=b)
    return b;
  
  return a;
}


static inline double pythag(double a, double b)
{
  double absa,absb;
  absa = fabs(a);
  absb = fabs(b);
  if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
  else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}


#define qa(i, j) (a[(i)*(n+1)+(j)])
#define qv(i, j) (v[(i)*(n+1)+(j)])
#define qu(i, j) (u[(i)*(n+1)+(j)])


static inline void svdcmp(double *a, int m, int n, double w[], double *v)
{
  int flag, i, its, j, jj, k, l, nm;
  double anorm, c, f, g, h, s, scale, x, y, z, *rv1;
  l = 0;
  nm = 0;
  rv1 = vector(1,n);
  g = scale = anorm = 0.0;
  for (i = 1;i <= n;i++) {
    l = i+1;
    rv1[i] = scale*g;
    g = s = scale=0.0;
    if (i <= m) {
      for (k = i;k <= m;k++) scale += fabs(qa(k,i));
      if (scale) {
	for (k  = i;k <= m;k++) {
	  qa(k,i) /= scale;
	  s += qa(k,i)*qa(k,i);
	}
	f = qa(i,i);
	g = -SIGN(sqrt(s),f);
	h = f*g-s;
	qa(i,i) = f-g;
	for (j = l;j <= n;j++) {
	  for (s = 0.0,k = i;k <= m;k++) s += qa(k,i)*qa(k,j);
	  f = s/h;
	  for (k = i;k <= m;k++) qa(k,j) += f*qa(k,i);
	}
	for (k = i;k <= m;k++) qa(k,i) *= scale;
      }
    }
    w[i] = scale *g;
    g = s = scale = 0.0;
    if(i <= m && i != n){
      for (k = l;k <= n;k++) scale += fabs(qa(i,k));
      if (scale) {
	for (k = l;k <= n;k++) {
	  qa(i,k) /= scale;
	  s += qa(i,k)*qa(i,k);
	}f = qa(i,l);
	g = -SIGN(sqrt(s),f);
	h = f*g - s;
	qa(i,l)=f-g;
	for (k = l;k <= n;k++) rv1[k] = qa(i,k)/h;
	for (j = l;j <= m;j++) {
	  for (s = 0.0,k = l;k <= n;k++) s += qa(j,k)*qa(i,k);
	  for (k = l;k <= n;k++) qa(j,k) += s*rv1[k];
	}
	for (k=l;k<=n;k++) qa(i,k) *= scale;
      }
    }
    anorm=FMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
  }
  for (i=n;i>=1;i--) {
    if(i<n){
      if (g) {
	for (j=l;j<=n;j++)
	  qv(j,i)=(qa(i,j)/qa(i,l))/g;
	for (j=l;j<=n;j++) {
	  for (s=0.0,k=l;k<=n;k++) s += qa(i,k)*qv(k,j);
	  for (k=l;k<=n;k++) qv(k,j) += s*qv(k,i);
	}
      }
      for (j=l;j<=n;j++) qv(i,j)=qv(j,i)=0.0;
    }
    qv(i,i)=1.0;
    g=rv1[i];
    l=i;
  }
  for (i=IMIN(m,n);i>=1;i--) {
    l=i+1;
    g=w[i];
    for (j=l;j<=n;j++) qa(i,j)=0.0;
    if (g) {
      g=1.0/g;
      for (j=l;j<=n;j++) {
	for (s=0.0,k=l;k<=m;k++) s += qa(k,i)*qa(k,j);
	f=(s/qa(i,i))*g;
	for (k=i;k<=m;k++) qa(k,j) += f*qa(k,i);
      }
      for (j=i;j<=m;j++) qa(j,i) *= g;
    } else for (j=i;j<=m;j++) qa(j,i)=0.0;
    ++qa(i,i);
  }
  for (k=n;k>=1;k--) {
    for (its=1;its<=30;its++) {
      flag=1;
      for (l=k;l>=1;l--) {
	nm=l-1;
	if ((double)(fabs(rv1[l])+anorm) == anorm) {
	  flag=0;
	  break;
	}
	if ((double)(fabs(w[nm])+anorm) == anorm) break;
      }
      if (flag) {
	c=0.0;
	s=1.0;
	for (i=l;i<=k;i++) {
	  f=s*rv1[i];
	  rv1[i]=c*rv1[i];
	  if ((double)(fabs(f)+anorm) == anorm) break;
	  g=w[i];
	  h=pythag(f,g);
	  w[i]=h;
	  h=1.0/h;
	  c=g*h;
	  s = -f*h;
	  for (j=1;j<=m;j++) {
	    y=qa(j,nm);
	    z=qa(j,i);
	    qa(j,nm)=y*c+z*s;
	    qa(j,i)=z*c-y*s;
	  }
	}
      }
      z=w[k];
      if (l == k) {
	if (z < 0.0) {
	  w[k] = -z;
	  for (j=1;j<=n;j++) qv(j,k) = -qv(j,k);
	}
	break;
      }
      if (its == 30) nrerror("No convergence in iterations.");
      x=w[l];
      nm=k-1;
      y=w[nm];
      g=rv1[nm];
      h=rv1[k];
      f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
      g = pythag(f,1.0);
      f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
      c=s=1.0;
      for (j=l;j<=nm;j++) {
	i=j+1;
	g=rv1[i];
	y=w[i];
	h=s*g;
	g=c*g;
	z=pythag(f,h);
	rv1[j]=z;
	c=f/z;
	s=h/z;
	f=x*c+g*s;
	g = g*c-x*s;
	h=y*s;
	y*=c;
	for (jj=1;jj<=n;jj++) {
	  x=qv(jj,j);
	  z=qv(jj,i);
	  qv(jj,j)=x*c+z*s;
	  qv(jj,i)=z*c-x*s;
	}
	z=pythag(f,h);
	w[j]=z;
	if (z) {
	  z=1.0/z;
	  c=f*z;
	  s=h*z;
	}
	f=c*g+s*y;
	x=c*y-s*g;
	for (jj=1;jj<=m;jj++) {
	  y=qa(jj,j);
	  z=qa(jj,i);
	  qa(jj,j)=y*c+z*s;
	  qa(jj,i)=z*c-y*s;
	}
      }
      rv1[l]=0.0;
      rv1[k]=f;
      w[k]=x;
    }
  }
  free_vector(rv1,1,n);
}


static inline void svbksb(double *u, double w[], double *v, int m, int n, double b[], double x[])
{
  int jj,j,i;
  double s,*tmp;

  tmp=vector(1,n);
  for (j=1;j<=n;j++) {    
    s=0.0;
    if (w[j]) {
      for (i=1;i<=m;i++) s += qu(i,j)*b[i];
      s /= w[j];
    }
    tmp[j]=s;
  }
  for (j=1;j<=n;j++) {
    s=0.0;
    for (jj=1;jj<=n;jj++) s += qv(j,jj)*tmp[jj];
    x[j]=s;
  }
  free_vector(tmp,1,n);
}


void LinearSolver::solve(int nbEquations, int nbUnknowns,
			 const double *equations,
			 const double *results,
			 double *unknowns)
{
  assert(nbEquations>0);
  assert(nbUnknowns>0);
  assert(nbEquations>=nbUnknowns);
  assert(equations!=0);
  assert(results!=0);
  assert(unknowns!=0);

  
  const int m=nbEquations;
  const int n=nbUnknowns;
  double *a=new double[(m+1)*(n+1)];
  double *b=new double[(m+1)];
  double *x=new double[(n+1)];
  double *v=new double[(n+1)*(n+1)];
  double *w=new double[(n+1)];

  int i;
  for (i=0; i<m; i++)
    for (int j=0; j<n; j++)
      qa(i+1, j+1)=equations[i*n+j];

  svdcmp(a, m, n, w, v);


  for (i=0; i<m; i++)
    b[i+1]=results[i];

   
  /*     double wmax=0.0;
     for(int j=1;j<=3;j++) if (w[j] > wmax) wmax=w[j];
     double wmin=wmax*1.0e-6;
     for(int j=1;j<=3;j++) if (w[j] < wmin) w[j]=0.0;
  */

  svbksb(a, w, v, m, n, b, x);

  for (i=0; i<n; i++)
    unknowns[i] = x[i+1];  

  delete [] a;
  delete [] b;
  delete [] x;
  delete [] v;
  delete [] w;
}

