#ifndef ABSTRACTCONFIGURATIONWIDGET_H
#define ABSTRACTCONFIGURATIONWIDGET_H

#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qcolor.h>

#include "../src/Interfaces/TextureInterface.h"

static const unsigned int COTEX_FONT_SIZE = 10;
static const unsigned int TEX_MARGIN = 10;
static const unsigned int TEX_SPACING = 20;
static const QString TEX_BASENAME = "Texture";
static const unsigned int TEX_ICON_WIDTH = 12;
static const unsigned int TEX_ICON_HEIGHT = 12;

/// The base class for all widgets able to set and create textures.
class AbstractConfigurationWidget : public QGroupBox {
  
  Q_OBJECT
  
public:

  /// The constructor
  AbstractConfigurationWidget (QWidget * parent = NULL);
  
  /// The destructor
  virtual ~AbstractConfigurationWidget () {}

  // ******** 
  // Services 
  // ********
  
  /// Creates a new texture with the current configuration of the object
  virtual TextureInterface * createTexture () const = 0;

  /// Creates the associated pixmap with the described texture 
  /// by the configuration of the object  
  virtual QPixmap createPixmap () const = 0;

protected:
  
  /// Completes the construction of the object
  virtual void placeUIElements (QLayout * baseLayout) = 0;
  
  /// Creates a button whose allows to choose a color
  QPushButton * createColorChooserButton (const QString & name, const QColor & color);
  
 /// Allows to update the color with the given color
  void updateButtonColor (QPushButton * button, const QColor & color);
 
  void selectColor (QPushButton * button, QColor & color);
  QString getCurrentTextureName () const;
  void setCurrentTextureName (QString & newName);

  /// Returns the common layout at all the inherited objects
  /**
   * This function is only used for the internal communication 
   * between the inherited objects.
   */
  QHBoxLayout * getBaseLayout () const;
  
private :

  QHBoxLayout * baseLayout; ///< the common layout at all the inherited objects 
  QLineEdit * nameLineEdit; ///< contains the name of the texture in creation
  
  /// Gives a size to a given widget
  void resizeWidget (QWidget * widget, QSize size);

  /// Creates a QPushButton with label for label, parent for parent and at the size size
  QPushButton * createPushButton (QString label, QWidget * parent, QSize size);
};
#endif // ABSTRACTCONFIGURATIONWIDGET_H
