#include <qlabel.h>
#include <qpainter.h>
#include <assert.h>
#include <qlabel.h>

#include "PerlinNoiseConfigurationWidget.h"

static const QString PERLIN_TITLE = "Perlin Noise";
static const QColor DEFAULT_COLOR1 = QColor (0, 0, 0);
static const QColor DEFAULT_COLOR2 = QColor (255, 255, 255);
static const double PERLIN_NOISE_DOUBLE_MIN = -50.0;
static const double PERLIN_NOISE_DOUBLE_MAX = 50.0;
static const int PERLIN_NOISE_DOUBLE_DECIMALES = 2;
static const double PERLIN_NOISE_DOUBLE_STEP = 0.01;
static const unsigned int PERLIN_MARGIN = 5;

PerlinNoiseConfigurationWidget::PerlinNoiseConfigurationWidget (QWidget * parent)
  : AbstractConfigurationWidget (parent) {
  placeUIElements (getBaseLayout ());
}


void PerlinNoiseConfigurationWidget::placeUIElements (QLayout * baseLayout) {
  assert (baseLayout != NULL);
  
  setTitle (PERLIN_TITLE);  
  chosenColor1 = DEFAULT_COLOR1;
  chosenColor2 = DEFAULT_COLOR2;
  
  QLabel * color1Label = new QLabel ("color1 :", this);
  color1Label->resize (color1Label->sizeHint ());
  colorChooserButton1 = createColorChooserButton ("", chosenColor1);
  connect (colorChooserButton1, SIGNAL (clicked ()), 
	   this, SLOT (appearColorDialog1 ()));
  QHBoxLayout * color1HBoxLayout = new QHBoxLayout ();
  color1HBoxLayout->setMargin (PERLIN_MARGIN);
  color1HBoxLayout->addWidget (color1Label);
  color1HBoxLayout->addWidget (colorChooserButton1);
  
  QLabel * color2Label = new QLabel ("color2 :", this);
  color2Label->resize (color2Label->sizeHint ());
  colorChooserButton2 = createColorChooserButton ("", chosenColor2);
  connect (colorChooserButton2, SIGNAL (clicked ()), 
	   this, SLOT (appearColorDialog2 ())); 
  QHBoxLayout * color2HBoxLayout = new QHBoxLayout ();
  color2HBoxLayout->setMargin (PERLIN_MARGIN);
  color2HBoxLayout->addWidget (color2Label);
  color2HBoxLayout->addWidget (colorChooserButton2);
   
  QLabel * bound1Label = new QLabel ("bound 1 : ", this);
  bound1SpinBox = new DoubleSpinBox (0.0, 2.0,
				     PERLIN_NOISE_DOUBLE_DECIMALES, 
				     PERLIN_NOISE_DOUBLE_STEP,
				     0.0,
				     this);
  QHBoxLayout * bound1HBoxLayout = new QHBoxLayout ();
  bound1HBoxLayout->addWidget (bound1Label);
  bound1HBoxLayout->addWidget (bound1SpinBox);

  QLabel * bound2Label = new QLabel ("bound 2 : ", this);
  bound2SpinBox = new DoubleSpinBox (0.0, 2.0,
				     PERLIN_NOISE_DOUBLE_DECIMALES, 
				     PERLIN_NOISE_DOUBLE_STEP,
				     0.0,
				     this);
  QHBoxLayout * bound2HBoxLayout = new QHBoxLayout ();
  bound2HBoxLayout->addWidget (bound2Label);
  bound2HBoxLayout->addWidget (bound2SpinBox);
  
  QLabel * phaseLabel = new QLabel ("phase : ", this);
  phaseSpinBox = new DoubleSpinBox (PERLIN_NOISE_DOUBLE_MIN, 
				    PERLIN_NOISE_DOUBLE_MAX,
				    PERLIN_NOISE_DOUBLE_DECIMALES, 
				    PERLIN_NOISE_DOUBLE_STEP,
				    0.0,
				    this);
  QHBoxLayout * phaseHBoxLayout = new QHBoxLayout ();
  phaseHBoxLayout->addWidget (phaseLabel);
  phaseHBoxLayout->addWidget (phaseSpinBox);

  QLabel * frequencyLabel = new QLabel ("frequency : ", this);
  frequencySpinBox = new DoubleSpinBox (PERLIN_NOISE_DOUBLE_MIN, 
					PERLIN_NOISE_DOUBLE_MAX,
					PERLIN_NOISE_DOUBLE_DECIMALES, 
					PERLIN_NOISE_DOUBLE_STEP,
					0.0,
					this);
  QHBoxLayout * frequencyHBoxLayout = new QHBoxLayout ();
  frequencyHBoxLayout->addWidget (frequencyLabel);
  frequencyHBoxLayout->addWidget (frequencySpinBox);

  QLabel * amplitudeLabel = new QLabel ("amplitude : ", this);
  amplitudeSpinBox = new DoubleSpinBox (PERLIN_NOISE_DOUBLE_MIN, 
					PERLIN_NOISE_DOUBLE_MAX,
					PERLIN_NOISE_DOUBLE_DECIMALES, 
					PERLIN_NOISE_DOUBLE_STEP,
					0.0,
					this);
  QHBoxLayout * amplitudeHBoxLayout = new QHBoxLayout ();
  amplitudeHBoxLayout->addWidget (amplitudeLabel);
  amplitudeHBoxLayout->addWidget (amplitudeSpinBox);

  perlinNoiseTypes = new QComboBox (false, this);
  perlinNoiseTypes->insertItem ("marble", MARBLENOISE);
  perlinNoiseTypes->insertItem ("wood", WOODNOISE);
  perlinNoiseTypes->insertItem ("cloud", CLOUDNOISE);
  perlinNoiseTypes->insertItem ("vscratch", V_SCRATCHED);
  perlinNoiseTypes->insertItem ("hscratch", H_SCRATCHED);

  QGridLayout * parametersGridLayout = new QGridLayout (4, 2);
  parametersGridLayout->addLayout (bound1HBoxLayout, 0, 0);
  parametersGridLayout->addLayout (bound2HBoxLayout, 1, 0);
  parametersGridLayout->addWidget (perlinNoiseTypes, 2, 0);
  parametersGridLayout->addLayout (phaseHBoxLayout, 0, 1);
  parametersGridLayout->addLayout (amplitudeHBoxLayout, 1, 1);
  parametersGridLayout->addLayout (frequencyHBoxLayout, 2, 1);
  parametersGridLayout->addLayout (color1HBoxLayout, 3, 0);
  parametersGridLayout->addLayout (color2HBoxLayout, 3, 1);

  QVBoxLayout * perlinLayout = new QVBoxLayout (this);
  perlinLayout->setMargin (PERLIN_MARGIN);
  perlinLayout->addLayout (baseLayout);
  perlinLayout->addLayout (parametersGridLayout);
  perlinLayout->activate (); 
}

// ******** 
// Services
// ********

TextureInterface * PerlinNoiseConfigurationWidget::createTexture () const {
  return new PerlinNoiseColorTexture (getCurrentTextureName (), 
				      chosenColor1,
				      chosenColor2,
				      phaseSpinBox->value (),
				      frequencySpinBox->value (),
				      amplitudeSpinBox->value (),
				      bound1SpinBox->value (),
				      bound2SpinBox->value (),
				      (NoiseType)perlinNoiseTypes->currentItem ());
}

QPixmap PerlinNoiseConfigurationWidget::createPixmap () const {
  QPixmap pix (TEX_ICON_WIDTH, TEX_ICON_HEIGHT);
  
  QPainter painter;
  painter.begin (&pix);
  unsigned int i, j;
  
  painter.setPen (QPen (chosenColor1, 1 , Qt::SolidLine));
  for (i = 0; i < TEX_ICON_WIDTH / 2; i++)
    for (j = 0; j < TEX_ICON_HEIGHT; j++) 
      painter.drawPoint (i, j);
  painter.setPen (QPen (chosenColor2, 1 , Qt::SolidLine));
  for (i = TEX_ICON_WIDTH / 2; i < TEX_ICON_WIDTH; i++)
    for (j = 0; j < TEX_ICON_HEIGHT; j++) 
      painter.drawPoint (i, j);
  
  painter.flush ();
  painter.end ();
    
  return pix;
}

void PerlinNoiseConfigurationWidget::conformTo (PerlinNoiseColorTexture * tex) {
  assert (tex != NULL);
  QString name = tex->getName ();
  
  chosenColor1 = tex->getColor1 ();
  chosenColor2 = tex->getColor2 ();
  updateButtonColor (colorChooserButton1, chosenColor1);
  updateButtonColor (colorChooserButton2, chosenColor2);
  setCurrentTextureName (name);
  
  phaseSpinBox->setValue (tex->getPhase ());
  frequencySpinBox->setValue (tex->getFrequency ());
  amplitudeSpinBox->setValue (tex->getAmplitude ());
  bound1SpinBox->setValue (tex->getBound1 ());
  bound2SpinBox->setValue (tex->getBound2 ());
  perlinNoiseTypes->setCurrentItem ((NoiseType)(tex->getNoiseType ()));
}


// *****
// Slots
// *****

void PerlinNoiseConfigurationWidget::appearColorDialog1 () {
  selectColor (colorChooserButton1, chosenColor1);
}

void PerlinNoiseConfigurationWidget::appearColorDialog2 () {
  selectColor (colorChooserButton2, chosenColor2);
}

